(* Author: Laura *)
(*AST*)

(*operators*)

type op		=	Add | Sub | Mult | Div | Equal | Neq | Less | Leq | Greater | Geq |
          			And | Or | Mod | StrEqual

type uop = Not

let string_of_op = function
	Add -> "+"
	| Sub -> "-"
	| Mult -> "*"
	| Div -> "/"
	| Equal -> "="
	| Neq -> "!="
	| Less -> "<"
	| Leq -> "<="
	| Greater -> ">"
	| Geq -> ">="
	| And -> "AND"
	| Or -> "OR"
	| Mod -> "%"
	| StrEqual -> "=="


let string_of_uop= function
	| Not -> "!"


(*data types*)

type typ	=	Bool | Int | Str | Piece | Void

let string_of_typ = function
	Bool -> "boolean"
	| Int -> "int"
	| Str -> "string"
	| Piece -> ""
	| Void -> ""

(* expressions are things that evaluate to a value or an action *)

type expr = 
	Id of string
	| Parenth of expr
	| BoolLiteral of bool
	| IntLiteral of int
	| StringLiteral of string
	| Binop of expr * op * expr 
	| Unop of uop * expr
  	| BoardAccess of expr*expr
  	| Access of string * string (* p.name *)
  	| Rand of expr * expr
  	| Empty

(*statements: things that you can do with expressions, things that dictate control flow.*)
type stmt = 
   Expr of expr
  | Assign of string * expr (* a {5}*)
  | VDecl of typ * string * expr
  | Call of string * expr list

  | Return of expr
  | If of expr * stmt list * stmt list
  (* for varName surrounding (int x, int y) *)
  | ForSurrounding of string * expr * expr * stmt list
  (*for varName in row(int rowNum)*)
  | ForRow of string * expr * stmt list
  (* for varName in col(int colNum) *)
  | ForCol of string * expr * stmt list
  (* for (int i, int j) surrounding (int x, int y) *)
  | ForSurroundingCoords of string * string * expr * expr * stmt list
  (* for varName(int min, int max) *)
  | ForMinMax of string * expr * expr * stmt list
  (* for varName(int min, int max), varName2(int min2, int max2) *)
  | ForNested of string * expr * expr * string * expr * expr * stmt list
  (* for (int x, int y) in board *)
  | ForBoard of string * string * stmt list
  | ForAll of string * stmt list

(*board definition*)
type board = {
	x: expr;
	y: expr;
	bg: expr;
}

(*a list of all players*)
type players = string list

(*a list that dictates the order in which players play*)
type turnOrder = string list

(*piece definition*)
type piece = {
	name: string;
	img: string;
	onTurn: stmt list;
	onClick: stmt list;
}

(*instructions for the setup of the board, rendered before any turns are completed*)
type setup = stmt list

(*conditions that end the game, specified as functions that return booleans. if not specified, we assume the functions indefinitely return 0.*)
type conditions = {
	win: stmt list;
	lose: stmt list;
	draw: stmt list;
	w: bool; l: bool; d: bool;
}

(*a program must comprised of all these parts*)
type program = board * players * turnOrder * piece list * setup * conditions
