/* Author: Laura */
%{
open Ast
%}
%token START BOARDDEF DIM IMG PLAYERS TURNORDER PIECE NAME ONTURN ONCLICK SETUP WIN LOSE DRAW IN SURROUNDING ROW COL ALL EMPTY RAND
%token LPAREN RPAREN LBRACE RBRACE LBRACK RBRACK COMMA
%token PLUS MINUS TIMES DIVIDE MOD ASSIGN NOT DECLARE
%token EQ NEQ LT LEQ GT GEQ TRUE FALSE AND OR STREQ PERIOD
%token RETURN IF ELSE FOR WHILE INT BOOL STR

%token <string> ID
%token <string> STRING_LITERAL
%token <int> INT_LITERAL
%token <bool> BOOL_LITERAL
%token EOF

%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN DECLARE
%left OR
%left AND
%left EQ NEQ STREQ
%left LT GT LEQ GEQ
%left PLUS MINUS 
%left TIMES DIVIDE MOD PERIOD
%right NOT

%start program
%type <Ast.program> program


%%


program:
  START LBRACE decl RBRACE EOF { $3 }


decl:
  board_decl players_decl turnOrder_decl piece_list setup conditions {$1, $2, $3, $4, $5, $6}

setup:
  SETUP LBRACE stmt_list RBRACE { $3 }

conditions:
  /*nothing*/ {{win=[]; lose=[]; draw=[]; w=false; l=false; d=false;}}
  | winCon  {{win=$1; lose=[]; draw=[]; w=true; l=false;d=false;}}
  | loseCon {{win=[]; lose=$1; draw=[];w=false;l=true;d=false;}}
  | drawCon {{win=[]; lose=[]; draw=$1;w=false;l=false;d=true;}}
  | winCon loseCon  {{win=$1; lose=$2; draw=[];w=true;l=true;d=false;}}
  | winCon drawCon  {{win=$1; lose=[]; draw=$2;w=true;l=false;d=true;}}
  | loseCon winCon  {{win=$2; lose=$1; draw=[];w=true;l=true;d=false;}}
  | loseCon drawCon {{win=[]; lose=$1; draw=$2; w=false;l=true;d=true;}}
  | drawCon winCon  {{win=$2; lose=[]; draw=$1;w=true;l=false;d=true;}}
  | drawCon loseCon {{win=[]; lose=$2; draw=$1;w=false;l=true;d=true;}}
  | winCon loseCon drawCon  {{win=$1; lose=$2; draw=$3;w=false;l=false;d=true;}} 
  | winCon drawCon loseCon  {{win=$1; lose=$3; draw=$2;w=true;l=true;d=true;}}
  | loseCon winCon drawCon  {{win=$2; lose=$1; draw=$3;w=true;l=true;d=true;}}
  | loseCon drawCon winCon  {{win=$3; lose=$1; draw=$2;w=true;l=true;d=true;}}
  | drawCon loseCon winCon  {{win=$3; lose=$2; draw=$1;w=true;l=true;d=true;}}
  | drawCon winCon loseCon  {{win=$2; lose=$3; draw=$1;w=true;l=true;d=true;}}


winCon:
  WIN LBRACE stmt_list RBRACE { $3 }

loseCon:
  LOSE LBRACE stmt_list RBRACE { $3 }

drawCon:
  DRAW LBRACE stmt_list RBRACE { $3 }

board_decl:
  BOARDDEF LBRACE DIM LBRACE expr COMMA expr RBRACE IMG LBRACE expr RBRACE RBRACE
  {
    {
      x = $5;
      y = $7;
      bg = $11;
    }
  }
    

players_decl:
  PLAYERS LBRACE string_list RBRACE {List.rev $3}

turnOrder_decl:
  TURNORDER LBRACE string_list RBRACE {List.rev $3}

piece_list:
  /*nothing */ {[]}
  | piece_list piece_decl {$2 :: $1}

piece_decl:
  PIECE LBRACE NAME LBRACE STRING_LITERAL RBRACE IMG LBRACE STRING_LITERAL RBRACE ONTURN LBRACE stmt_list RBRACE ONCLICK LBRACE stmt_list RBRACE RBRACE
  {
    {
      name = $5;
      img = $9;
      onTurn = $13;
      onClick = $17;
    }

  }


string_list:
  /* nothing */ {[]}
  | STRING_LITERAL {$1::[]}
  | string_list COMMA STRING_LITERAL  {$3 :: $1}

stmt_list:
    /* nothing */  { [] }
  | stmt_list stmt { $2 :: $1 }

stmt:
  | expr { Expr $1 }
  | RETURN LBRACE expr RBRACE { Return $3 }
  | IF LPAREN expr RPAREN LBRACE stmt_list RBRACE %prec NOELSE { If($3, $6, []) }
  | IF LPAREN expr RPAREN LBRACE stmt_list RBRACE ELSE LBRACE stmt_list RBRACE    { If($3, $6, $10) }
  | FOR ID IN ROW LPAREN expr RPAREN LBRACE stmt_list RBRACE {ForRow($2,$6,$9)}
  | FOR ID IN COL LPAREN expr RPAREN LBRACE stmt_list RBRACE {ForCol($2,$6,$9)}
  | FOR ID SURROUNDING LPAREN expr COMMA expr RPAREN LBRACE stmt_list RBRACE {ForSurrounding($2, $5, $7, $10)}
  | FOR LPAREN ID COMMA ID RPAREN SURROUNDING LPAREN expr COMMA expr RPAREN LBRACE stmt_list RBRACE {ForSurroundingCoords($3, $5, $9, $11, $14)}
  | FOR ID LPAREN expr COMMA expr RPAREN LBRACE stmt_list RBRACE {ForMinMax($2,$4,$6,$9)}
  | FOR LPAREN ID COMMA ID RPAREN IN BOARDDEF LBRACE stmt_list RBRACE {ForBoard($3, $5,$10)}
  | FOR ID LPAREN expr COMMA expr RPAREN COMMA ID LPAREN expr COMMA expr RPAREN LBRACE stmt_list RBRACE {ForNested($2, $4, $6,$9,$11,$13,$16)}
  | FOR ALL ID IN BOARDDEF LBRACE stmt_list RBRACE {ForAll($3,$7)}
  | ASSIGN ID LBRACE expr RBRACE { Assign($2, $4) }
  | ID LBRACE actuals_opt RBRACE { Call($1, $3) }
  | DECLARE typ ID LBRACE expr RBRACE {VDecl($2, $3, $5)}

expr:
    STRING_LITERAL               { StringLiteral($1) }
  | BOOL_LITERAL                 { BoolLiteral ($1) }
  | INT_LITERAL                  { IntLiteral ($1)}
  | ID                           { Id($1) }
  | EMPTY                        {Empty}
  | LPAREN expr RPAREN       { Parenth($2) }
  | BOARDDEF LBRACK expr RBRACK LBRACK expr RBRACK {BoardAccess($3,$6)}
  | expr PLUS   expr { Binop($1, Add,   $3) }
  | expr MINUS  expr { Binop($1, Sub,   $3) }
  | expr TIMES  expr { Binop($1, Mult,  $3) }
  | expr DIVIDE expr { Binop($1, Div,   $3) }
  | expr MOD    expr { Binop($1, Mod,   $3) }
  | expr EQ     expr { Binop($1, Equal, $3) }
  | expr STREQ  expr { Binop($1, StrEqual,$3)}
  | expr NEQ    expr { Binop($1, Neq,   $3) }
  | expr LT     expr { Binop($1, Less,  $3) }
  | expr LEQ    expr { Binop($1, Leq,   $3) }
  | expr GT     expr { Binop($1, Greater, $3) }
  | expr GEQ    expr { Binop($1, Geq,   $3) }
  | expr AND    expr { Binop($1, And,   $3) }
  | expr OR     expr { Binop($1, Or,    $3) }
  | NOT         expr { Unop (Not, $2)}
  | ID PERIOD ID {Access($1,$3)}
  | RAND LBRACE expr COMMA expr RBRACE {Rand($3,$5)}

typ:
  INT {Int}
  | BOOL {Bool}
  | STR  {Str}
  

actuals_opt:
    /* nothing */ { [] }
  | actuals_list  { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }
