(* Author: Laura *)
open Ast


type var_ref = string
type var_new = string
type func_name = string 
type obj_field = string

type expr_detail =
	| Parenth of expr_with_type
	| Id of var_ref
	| BoolLiteral of bool
	| IntLiteral of int
	| StringLiteral of string
	| Binop of expr_with_type * Ast.op * expr_with_type
	| Unop of Ast.uop * expr_with_type
	| Access of var_ref * obj_field
	| BoardAccess of expr_with_type * expr_with_type
	| Rand of expr_with_type * expr_with_type
	| Empty
 	and expr_with_type = Ast.typ * expr_detail



(*statements: things that you can do with expressions, things that dictate control flow.*)
type stmt_detail = 
   Expr of expr_with_type
  | Return of expr_with_type
  | If of expr_with_type * stmt_detail list * stmt_detail list
  (* for varName surrounding (int x, int y) *)
  | ForSurrounding of var_new * expr_with_type * expr_with_type * stmt_detail list
  (*for varName in row(int rowNum)*)
  | ForRow of var_new * expr_with_type * stmt_detail list
  (* for varName in col(int colNum) *)
  | ForCol of var_new * expr_with_type * stmt_detail list
  (* for (int i, int j) surrounding (int x, int y) *)
  | ForSurroundingCoords of var_new * var_new * expr_with_type * expr_with_type * stmt_detail list
  (* for varName(int min, int max) *)
  | ForMinMax of var_new * expr_with_type * expr_with_type * stmt_detail list
  (* for varName(int min, int max), varName2(int min2, int max2) *)
  | ForNested of var_new * expr_with_type * expr_with_type * var_new * expr_with_type * expr_with_type * stmt_detail list
  (* for (int x, int y) in board *)
  | ForBoard of var_new * var_new * stmt_detail list
  | ForAll of var_new * stmt_detail list
  | Assign of var_ref * expr_with_type
  | VDecl of var_new * expr_with_type
  | Call of func_name * expr_with_type list

type board_typed = {
	x: expr_with_type;
	y: expr_with_type;
	img: expr_with_type; 
}

(*piece definition*)
type piece_typed = {
	name: string;
	img: string;
	onTurn_typed: stmt_detail list;
	onClick_typed: stmt_detail list;
}

(*instructions for the setup of the board, rendered before any turns are completed*)
type setup_typed = stmt_detail list

(*conditions that end the game, specified as functions that return booleans. if not specified, we assume the functions indefinitely return 0.*)
type conditions_typed = {
	win_typed: stmt_detail list;
	lose_typed: stmt_detail list;
	draw_typed: stmt_detail list;
}
