(* Author: Laura *)
{ open Parser }

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
  | "/*"     { comment lexbuf } 
| "game"{ START }
| "def"	{ DECLARE }
| "set" { ASSIGN }
| "board"	{BOARDDEF}
| "dimensions"	{ DIM }
| "image"	{ IMG }
| "players" { PLAYERS }
| "turnOrder" {TURNORDER}
| "piece"	{ PIECE }
| "setup"	{ SETUP }
| "winCondition" { WIN }
| "loseCondition" { LOSE }
| "drawCondition"	{ DRAW }
| "name"	{ NAME }
| "onTurn"	{ ONTURN }
| "onClick"	{ ONCLICK }
| "rand"    { RAND }
| "if"		{ IF }
| "else"	{ ELSE }
| "for"		{ FOR }
| "in"		{ IN }
| "all"		{ ALL }
| "surrounding" {SURROUNDING}
| "row"		{ROW}
| "col"		{COL}
| "return"	{ RETURN }
| "int"		{INT}
| "str"		{STR}
| "bool"	{BOOL}
| '{'	{ LBRACE }			
| '}'	{ RBRACE }
| ','	{ COMMA }
| '('	{ LPAREN }			
| ')'	{ RPAREN }
| '['	{ LBRACK }			
| ']'	{ RBRACK }
| '+'	{ PLUS }				
| '-'	{ MINUS }			
| '*'	{ TIMES }
| '/'	{ DIVIDE }				
| '%'	{ MOD }
| "empty" {EMPTY}
| "=="	{ STREQ }				
| "="	{ EQ }
| "!="	{ NEQ }
| '<'	{ LT }					
| "<="	{ LEQ }
| '>'	{ GT }					
| ">="	{ GEQ }
| "AND"	{ AND }			
| "OR"	{ OR }
| "!"	{ NOT }
| "."	{ PERIOD }
| "true" | "false" as lxm { BOOL_LITERAL(bool_of_string lxm) }
| ['0'-'9']+ as lxm { INT_LITERAL(int_of_string lxm) }
| '"'('\\'_|[^'"'])*'"' as lxm { STRING_LITERAL(lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof 	{ EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }
